# -*- coding:utf-8 -*-
from django.contrib import admin
from django.utils.text import capfirst
from django.urls import reverse, NoReverseMatch
import six
from django.apps import apps
from django.conf import settings


def extra_menus(request):
    menus = []
    for menu in settings.EXTRA_MENUS:
        element = menu.copy()
        element['options'] = []
        for o in menu['options']:
            if request.user.has_perm(o['perm']):
                element['options'].append(o)
        if len(element['options']) > 0:
            menus.append(element)
    return {'extra_menus': menus}


site = admin.site


def applist(request):
    app_dict = {}
    user = request.user
    for model, model_admin in site._registry.items():
        app_label = model._meta.app_label
        has_module_perms = user.has_module_perms(app_label)

        if has_module_perms:
            perms = model_admin.get_model_perms(request)

            # Check whether user has any perm for this module.
            # If so, add the module to the model_list.
            if True in perms.values():
                info = (app_label, model._meta.model_name)
                model_dict = {
                    'name': capfirst(model._meta.verbose_name_plural),
                    'object_name': model._meta.object_name,
                    'perms': perms,
                }
                if perms.get('change', False):
                    try:
                        model_dict['admin_url'] = reverse(
                            'admin:%s_%s_changelist'
                            % info, current_app=site.name)
                    except NoReverseMatch:
                        pass
                if perms.get('add', False):
                    try:
                        model_dict['add_url'] = reverse(
                            'admin:%s_%s_add' % info, current_app=site.name)
                    except NoReverseMatch:
                        pass
                if app_label in app_dict:
                    app_dict[app_label]['models'].append(model_dict)
                else:
                    app_dict[app_label] = {
                        'name': apps.get_app_config(app_label).verbose_name,
                        'app_label': app_label,
                        'app_url': reverse('admin:app_list',
                        kwargs={'app_label': app_label}, current_app=site.name),
                        'has_module_perms': has_module_perms,
                        'models': [model_dict],
                    }

    # Sort the apps alphabetically.
    app_list = list(six.itervalues(app_dict))
    app_list.sort(key=lambda x: x['name'].lower())

    # Sort the models alphabetically within each app.
    for app in app_list:
        app['models'].sort(key=lambda x: x['name'])
    return {'app_list': app_list}




from django.contrib import admin
from import_export.admin import ImportExportModelAdmin
from django.contrib.admin import site
import adminactions.actions as actions
actions.add_to_site(site)


class entidad_admin(ImportExportModelAdmin):
    list_display = ('code', 'name')
    ordering = ('-code',)



class documento_admin(admin.ModelAdmin):
    ordering = ('-number',)
    date_hierarchy = 'date'
    change_form_template = "print/document.html"
    printed_readonly_fields = ()
    only_superuserfields = ('date', 'number')
    def get_readonly_fields(self, request, obj=None):
        if request.user.is_superuser:
            return []
        if obj and obj.impreso:
            return self.printed_readonly_fields
        elif obj and obj.cliente:
            return self.only_superuserfields + ('name', 'phone', 'nit', 'address')
        else:
            return self.only_superuserfields



class document_detail(admin.TabularInline):
    extra = 0
    classes = ('grp-collapse grp-open',)
    exclude = ('blog', )
    inline_classes = ('grp-collapse grp-open',)
    printed_readonly_fields = []
    def get_readonly_fields(self, request, obj=None):
        if not request.user.is_superuser and obj:
            if obj.impreso:
                return self.printed_readonly_fields
        return self.readonly_fields


from django.conf.urls import url
from .ajax import *

# app_label = "grappelli_extras" fixme check before use app label

urlpatterns = [
    url(r'^get_object/', get_object, name="ajax_getObject"),
    url(r'^object_update/', object_update, name="ajax_ObjectUpdate"),
    url(r'^object_view/', object_view, name="ajax_ObjectView"),
    url(r'^get_collection/', get_collection, name="ajax_getCollection"),
    url(r'^get_datatables/', get_datatables, name="ajax_getDataTables"),
    url(r'^autocomplete/', autocomplete, name="ajax_autocomplete"),
    url(r'^object_execute/', object_execute, name="ajax_objectExecute"),
    url(r'^get_html_form/', get_html_form, name="ajax_getHtmlForm"),
]



from django.http.response import HttpResponse
from django.forms.models import model_to_dict
from django.db.models import Q
import json
import operator
from django.core.mail import EmailMessage


def send_email(asunto="", texto="", correo=""):
    for e in correo.split(','):
        email = EmailMessage(asunto, texto,
                             to=[e],
                             )
        email.content_subtype = "html"
        email.send()


def find_entidad(sentence, filters=None):
    predicates = []
    if filters:
        predicates = filters
    for word in sentence.split(" "):
        predicates.append(('name__icontains', word))
    return [Q(x) for x in predicates]


def autocomplete_entidad(instance, request, filters=None):
    data = []
    if request.is_ajax:
        model = type(instance)
        result = []
        term = request.GET.get('term', None)
        if term:
            qs = model.objects.filter(reduce(operator.and_, find_entidad(term, filters)))
            for obj in qs:
                obj_json = {}
                obj_json['label'] = obj.name
                obj_json['value'] = obj.name
                obj_json['obj'] = obj.to_json()
                result.append(obj_json)
        data = json.dumps(result)
    return HttpResponse(data, content_type='application/json')


def entidad_to_json(instance, request):
    if request.is_ajax:
        model = type(instance)
        result = []
        id = request.POST.get('id', None)
        obj = model.objects.get(id=id)
        if obj:
            try:
                result = obj.to_json()
            except:
                result = model_to_dict(obj)
            data = json.dumps(result)
    else:
        data = 'fail'
    return HttpResponse(data, content_type='application/json')



# coding: utf8
from __future__ import unicode_literals

from django.db import models
from django.db.models import Max
from django.forms.models import model_to_dict


def get_code(entity, length=4):
        model = type(entity)
        code = ''
        sets = model.objects.filter(code__isnull=False)
        if sets:
            maxi = str(sets.aggregate(Max('code'))['code__max'])
            if maxi:
                consecutive = list(range(1, int(maxi)))
                busy = list(sets.values_list('code', flat=True))
                n = 0
                for l in busy:
                    busy[n] = int(str(l))
                    n += 1
                available = list(set(consecutive) - set(busy))
                if len(available) > 0:
                    code = min(available)
                else:
                    code = max(busy) + 1
        else:
            code = 1
        return str(code).zfill(length)


class base(models.Model):

    def __iter__(self):
        for field_name in self._meta.fields():
            try:
                value = getattr(self, field_name)
            except:
                value = None
            yield (field_name, value)

    def __getitem__(self, fieldname):
        try:
            return getattr(self, fieldname)
        except:
            return None

    def __setitem__(self, fieldname, value):
        try:
            return setattr(self, fieldname, value)
        except:
            return None

    def to_json(self):
        o = model_to_dict(self)
        o['app_label'] = self._meta.app_label
        o['model'] = self._meta.object_name.lower()
        return o

    class Meta:
        abstract = True


class BaseEntity(base):
    code = models.CharField(max_length=25, null=True, blank=True,
        verbose_name="code")
    name = models.CharField(max_length=100, verbose_name="name")
    active = models.BooleanField(default=True)

    def save(self, *args, **kwargs):
        if self.code is None or self.code == '':
            self.code = get_code(self)
        super().save()

    def __str__(self):
        if self.code and self.name:
            return "%s %s" % (str(self.code), str(self.name))
        elif self.name:
            return self.name
        elif self.code:
            return str(self.code)
        else:
            return ''

    class Meta:
        abstract = True
        ordering = ['name']


class Entity(BaseEntity):

    @staticmethod
    def autocomplete_search_fields():
        return ("code__iexact", "name__icontains",)

    class Meta:
        abstract = True
        ordering = ['name']


class Document(base):
    firts_status = "Draft"
    date = models.DateField(null=True, blank=True)
    number = models.PositiveIntegerField(null=True, blank=True)
    blog = models.TextField(max_length=5000, verbose_name="Correos e Información Relacionada", null=True, blank=True)
    printed = models.BooleanField(default=False)

    def printed_number(self, zeros=4):
        return str(self.number).zfill(zeros)

    def doc_name(self):
        if not self.number and not self.date:
            return self.firts_status
        else:
            return "%s # %s" % (self._meta.verbose_name, self.printed_number())

    doc_name.short_description = "Document"

    def __unicode__(self):
        return self.doc_name()

    class Meta:
        abstract = True



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function
from setuptools import setup, find_packages
import os
import re
import sys


VERSION_FILE = 'grappelli_extras/__init__.py'
version_text = open(VERSION_FILE, "rt").read()
VSRE = r"^__version__ = ['\"]([^'\"]*)['\"]"
mo = re.search(VSRE, version_text, re.M)
if mo:
    version = mo.group(1)
    print (version)
else:
    raise RuntimeError(
        "Unable to find version string in %s." % (VERSION_FILE,))

name = 'django-grappelli-extras'
package = 'grappelli_extras'
description = 'Ajax, Extensions and Extras for Grappelli Admin interface'
url = 'https://github.com/xangcastle/grappelli_extras/'
author = 'Cesar Abel Ramirez'
author_email = 'xangcastle@gmail.com'
license = 'BSD'
install_requires = [
    'django-grappelli>=2.11.1',
    'django-import-export>=1.0.1',
    'django-adminactions>=1.5 ',
]

if sys.argv[-1] == 'publish':
    args = {'version': version}
    os.system("python setup.py sdist bdist_wheel")
    os.system("twine upload dist/django-grappelli-extras-%(version)s*" % args)
    print("Creating git tag the version now:")
    os.system("git tag -a %(version)s -m 'version %(version)s'" % args)
    os.system("git push --tags")
    sys.exit()


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname), "r").read()


setup(
    name=name,
    version=version,
    url=url,
    author=author,
    author_email=author_email,
    license=license,
    packages=['grappelli_extras'],
    include_package_data=True,
    description=description,
    long_description=read('README.md'),
    long_description_content_type="text/markdown",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries",
        "Topic :: Utilities",
        "Environment :: Web Environment",
        "Framework :: Django",
    ],
    install_requires=install_requires
)



from django.shortcuts import render
from .utils import Filter

def to_print(request):
    app_label = request.GET.get('app_label')
    model = request.GET.get('model')
    instance = Filter(app_label=app_label, model_name=model
                      ).get_instance(int(request.GET.get('id')))
    instance.impreso = True
    instance.save()
    data = {'obj': instance}
    return render(request, "print/%s/%s.html" % (app_label, model), data)